// FontPalette6.cpp by katahiromz
// This file is public domain software (PDS).
#include <windows.h>
#include <windowsx.h>
#include <cstdio>

HINSTANCE g_hInstance;
BOOL g_bUsePt = TRUE;

typedef struct PITCH_INFO
{
    LPCTSTR Name;
    BYTE Pitch;
} PITCH_INFO;

PITCH_INFO g_pitch_table[] =
{
    { TEXT("DEFAULT_PITCH"), DEFAULT_PITCH },
    { TEXT("FIXED_PITCH"), FIXED_PITCH },
    { TEXT("VARIABLE_PITCH"), VARIABLE_PITCH }
};
INT g_pitch_count = INT(sizeof(g_pitch_table) / sizeof(g_pitch_table[0]));

BYTE PitchFromName(LPCTSTR Name)
{
    for (INT i = 0; i < g_pitch_count; ++i)
    {
        if (lstrcmpi(Name, g_pitch_table[i].Name) == 0)
            return g_pitch_table[i].Pitch;
    }
    return DEFAULT_PITCH;
}

// NOTE: TMPF_FIXED_PITCH is confusing
#define _TMPF_VAR_PITCH TMPF_FIXED_PITCH

LPCTSTR NameFromTMPitch(BYTE Pitch)
{
    if (Pitch & _TMPF_VAR_PITCH)
        return TEXT("VARIABLE_PITCH");
    else
        return TEXT("FIXED_PITCH");
}

typedef struct CHARSET_INFO
{
    LPCTSTR Name;
    BYTE    CharSet;
} CHARSET_INFO;

CHARSET_INFO g_charset_table[] =
{
    { TEXT("ANSI_CHARSET"), ANSI_CHARSET },
    { TEXT("DEFAULT_CHARSET"), DEFAULT_CHARSET },
    { TEXT("SYMBOL_CHARSET"), SYMBOL_CHARSET },
    { TEXT("SHIFTJIS_CHARSET"), SHIFTJIS_CHARSET },
    { TEXT("HANGEUL_CHARSET"), HANGEUL_CHARSET },
    { TEXT("HANGUL_CHARSET"), HANGUL_CHARSET },
    { TEXT("GB2312_CHARSET"), GB2312_CHARSET },
    { TEXT("CHINESEBIG5_CHARSET"), CHINESEBIG5_CHARSET },
    { TEXT("OEM_CHARSET"), OEM_CHARSET },
    { TEXT("JOHAB_CHARSET"), JOHAB_CHARSET },
    { TEXT("HEBREW_CHARSET"), HEBREW_CHARSET },
    { TEXT("ARABIC_CHARSET"), ARABIC_CHARSET },
    { TEXT("GREEK_CHARSET"), GREEK_CHARSET },
    { TEXT("TURKISH_CHARSET"), TURKISH_CHARSET },
    { TEXT("VIETNAMESE_CHARSET"), VIETNAMESE_CHARSET },
    { TEXT("THAI_CHARSET"), THAI_CHARSET },
    { TEXT("EASTEUROPE_CHARSET"), EASTEUROPE_CHARSET },
    { TEXT("RUSSIAN_CHARSET"), RUSSIAN_CHARSET },
    { TEXT("MAC_CHARSET"), MAC_CHARSET },
    { TEXT("BALTIC_CHARSET"), BALTIC_CHARSET }
};
INT g_charset_count = INT(sizeof(g_charset_table) / sizeof(g_charset_table[0]));

BYTE CharSetFromName(LPCTSTR Name)
{
    for (INT i = 0; i < g_charset_count; ++i)
    {
        if (lstrcmpi(g_charset_table[i].Name, Name) == 0)
            return g_charset_table[i].CharSet;
    }
    return DEFAULT_CHARSET;
}

LPCTSTR NameFromCharSet(BYTE bCharSet)
{
    for (INT i = 0; i < g_charset_count; ++i)
    {
        if (bCharSet == g_charset_table[i].CharSet)
            return g_charset_table[i].Name;
    }
    return TEXT("");
}

LPCTSTR NameFromWeight(LONG Weight)
{
    if (Weight == 0) return TEXT("FW_DONTCARE");
    if (Weight <= FW_THIN) return TEXT("FW_THIN");
    if (Weight <= FW_EXTRALIGHT) return TEXT("FW_EXTRALIGHT");
    if (Weight <= FW_ULTRALIGHT) return TEXT("FW_ULTRALIGHT");
    if (Weight <= FW_LIGHT) return TEXT("FW_LIGHT");
    if (Weight <= FW_NORMAL) return TEXT("FW_NORMAL");
    if (Weight <= FW_REGULAR) return TEXT("FW_REGULAR");
    if (Weight <= FW_MEDIUM) return TEXT("FW_MEDIUM");
    if (Weight <= FW_SEMIBOLD) return TEXT("FW_SEMIBOLD");
    if (Weight <= FW_DEMIBOLD) return TEXT("FW_DEMIBOLD");
    if (Weight <= FW_BOLD) return TEXT("FW_BOLD");
    if (Weight <= FW_EXTRABOLD) return TEXT("FW_EXTRABOLD");
    if (Weight <= FW_ULTRABOLD) return TEXT("FW_ULTRABOLD");
    if (Weight <= FW_HEAVY) return TEXT("FW_HEAVY");
    if (Weight <= FW_BLACK) return TEXT("FW_BLACK");
    return TEXT("(unknown weight)");
}

const TCHAR *GetFontSummary(HDC hDC, LOGFONT *plf, TEXTMETRIC *ptm, OUTLINETEXTMETRIC *potm)
{
    static TCHAR s_sz[MAX_PATH];
    TCHAR sz[MAX_PATH];
    LPCTSTR psz = NULL;

    if (potm->otmSize)
    {
        psz = LPCTSTR(LPBYTE(potm) + WORD(potm->otmpFaceName));
    }
    if (psz && psz[0])
        lstrcpy(s_sz, psz);
    else if (plf->lfFaceName[0])
        lstrcpy(s_sz, plf->lfFaceName);
    else
        lstrcpy(s_sz, TEXT("(unknown)"));

    INT Height = ptm->tmHeight - ptm->tmInternalLeading;
    INT nPointSize = MulDiv(Height, 72, GetDeviceCaps(hDC, LOGPIXELSY));
    wsprintf(sz, TEXT(", %dpt"), nPointSize);
    lstrcat(s_sz, sz);

    LPCTSTR pszCharSet = NameFromCharSet(ptm->tmCharSet);
    if (pszCharSet[0])
    {
        wsprintf(sz, TEXT(", %s"), pszCharSet);
        lstrcat(s_sz, sz);
    }

    LPCTSTR pszPitch = NameFromTMPitch(ptm->tmPitchAndFamily);
    if (pszPitch[0])
    {
        wsprintf(sz, TEXT(", %s"), pszPitch);
        lstrcat(s_sz, sz);
    }

    LPCTSTR pszBold = NameFromWeight(ptm->tmWeight);
    if (lstrcmpi(pszBold, TEXT("FW_NORMAL")) != 0 &&
        lstrcmpi(pszBold, TEXT("FW_REGULAR")) != 0)
    {
        wsprintf(sz, TEXT(", %s"), pszBold);
        lstrcat(s_sz, sz);
    }
    if (ptm->tmItalic)
    {
        lstrcat(s_sz, TEXT(", Italic"));
    }
    if (ptm->tmUnderlined)
    {
        lstrcat(s_sz, TEXT(", Underline"));
    }
    if (ptm->tmStruckOut)
    {
        lstrcat(s_sz, TEXT(", StrikeOut"));
    }
    return s_sz;
}

extern "C"
int CALLBACK EnumFontFamExProc(
    const ENUMLOGFONTEX *   lpelf,
    const TEXTMETRIC *      lptm,
    DWORD                   FontType,
    LPARAM                  lParam)
{
    HWND hwnd = HWND(lParam);
    LPCTSTR FaceName = lpelf->elfLogFont.lfFaceName;
    if (FaceName[0] == TEXT('@'))
        return TRUE;

    TCHAR szScript[LF_FACESIZE];
    GetDlgItemText(hwnd, cmb5, szScript, LF_FACESIZE);

    INT Index;
    if (szScript[0] == 0 || lstrcmp(szScript, lpelf->elfScript) == 0)
    {
        Index = SendDlgItemMessage(hwnd, cmb1, CB_FINDSTRINGEXACT, 0, LPARAM(FaceName));
        if (Index == CB_ERR)
            SendDlgItemMessage(hwnd, cmb1, CB_ADDSTRING, 0, LPARAM(FaceName));
    }
    if (szScript[0] == 0)
    {
        Index = SendDlgItemMessage(hwnd, cmb5, CB_FINDSTRINGEXACT, 0, LPARAM(lpelf->elfScript));
        if (Index == CB_ERR)
            SendDlgItemMessage(hwnd, cmb5, CB_ADDSTRING, 0, LPARAM(lpelf->elfScript));
    }

    return TRUE;
}

VOID UpdateFontList(HWND hwnd)
{
    LOGFONT lf;
    HDC hDC = CreateCompatibleDC(NULL);
    ZeroMemory(&lf, sizeof(lf));
    lf.lfCharSet = DEFAULT_CHARSET;
    SendDlgItemMessage(hwnd, cmb1, CB_RESETCONTENT, 0, 0);
    EnumFontFamiliesEx(hDC, &lf, FONTENUMPROC(EnumFontFamExProc), LPARAM(hwnd), 0);
    DeleteDC(hDC);

    if (SendDlgItemMessage(hwnd, cmb5, CB_GETCURSEL, 0, 0) == CB_ERR)
    {
        static const TCHAR sz[] = TEXT("");
        SendDlgItemMessage(hwnd, cmb5, CB_ADDSTRING, 0, LPARAM(sz));
    }
}

BOOL OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
    UpdateFontList(hwnd);

    SetDlgItemInt(hwnd, edt1, 18, FALSE);

    for (INT i = 0; i < g_charset_count; ++i)
    {
        SendDlgItemMessage(hwnd, cmb2, CB_ADDSTRING, 0, (LPARAM)g_charset_table[i].Name);
    }
    SendDlgItemMessage(hwnd, cmb2, CB_SETCURSEL, 1, 0);

    for (INT i = 0; i < g_pitch_count; ++i)
    {
        SendDlgItemMessage(hwnd, cmb3, CB_ADDSTRING, 0, (LPARAM)g_pitch_table[i].Name);
    }
    SendDlgItemMessage(hwnd, cmb3, CB_SETCURSEL, 0, 0);

    SendDlgItemMessage(hwnd, cmb4, CB_ADDSTRING, 0, (LPARAM)TEXT("pt"));
    SendDlgItemMessage(hwnd, cmb4, CB_ADDSTRING, 0, (LPARAM)TEXT("px"));
    SendDlgItemMessage(hwnd, cmb4, CB_SETCURSEL, 0, 0);

    return TRUE;
}

void OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
    switch (id)
    {
    case cmb1:
    case cmb2:
    case cmb3:
    case cmb4:
        switch (codeNotify)
        {
        case CBN_EDITCHANGE: case CBN_SELENDOK:
            InvalidateRect(hwnd, NULL, TRUE);
            break;
        }
        break;
    case cmb5:
        switch (codeNotify)
        {
        case CBN_EDITCHANGE: case CBN_SELENDOK:
            UpdateFontList(hwnd);
            InvalidateRect(hwnd, NULL, TRUE);
            break;
        }
        break;
    case edt1:
        if (codeNotify == EN_CHANGE)
        {
            InvalidateRect(hwnd, NULL, TRUE);
        }
        break;
    case IDOK:
        EndDialog(hwnd, IDOK);
        break;
    case IDCANCEL:
        EndDialog(hwnd, IDCANCEL);
        break;
    }
}

void OnPaint(HWND hwnd)
{
    PAINTSTRUCT ps;
    HDC hDC;
    HFONT hGuiFont, hFont;
    LOGFONT lf, lfReal;
    TCHAR szCharSet[32], szPitch[32], texts[5][128];
    INT Index, Size;

    for (Index = 0; Index < 5; ++Index)
    {
        LoadString(g_hInstance, Index + 1, texts[Index], 128);
    }

    hGuiFont = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
    GetObject(hGuiFont, sizeof(lf), &lf);
    lf.lfHeight = 12;
    hGuiFont = CreateFontIndirect(&lf);

    GetDlgItemText(hwnd, cmb2, szCharSet, 32);
    BYTE bCharSet = CharSetFromName(szCharSet);

    GetDlgItemText(hwnd, cmb3, szPitch, 32);
    BYTE bPitch = PitchFromName(szPitch);

    ZeroMemory(&lf, sizeof(lf));
    lf.lfCharSet = bCharSet;
    lf.lfPitchAndFamily = bPitch | FF_DONTCARE;

    if (SendDlgItemMessage(hwnd, cmb4, CB_GETCURSEL, 0, 0) == 0)
        g_bUsePt = TRUE;
    else
        g_bUsePt = FALSE;

    hDC = BeginPaint(hwnd, &ps);
    if (hDC)
    {
        GetDlgItemText(hwnd, cmb1, lf.lfFaceName, LF_FACESIZE);
        Size = GetDlgItemInt(hwnd, edt1, NULL, TRUE);
        if (g_bUsePt)
            lf.lfHeight = -MulDiv(Size, GetDeviceCaps(hDC, LOGPIXELSY), 72);
        else
            lf.lfHeight = -Size;

        Index = 0;
        LPCTSTR psz;
        HGDIOBJ hFontOld;
        TEXTMETRIC tm;
        BYTE MetricBuffer[1024];
        LPOUTLINETEXTMETRIC potm = LPOUTLINETEXTMETRIC(MetricBuffer);
        INT X = 15, Y = 50;
        INT Height;

        lf.lfWeight = FW_NORMAL;
        lf.lfItalic = FALSE;
        lf.lfUnderline = FALSE;
        lf.lfStrikeOut = FALSE;
        hFont = CreateFontIndirect(&lf);
        if (hFont)
        {
            hFontOld = SelectObject(hDC, hFont);
            TextOut(hDC, X, Y, texts[Index], lstrlen(texts[Index]));
            GetTextMetrics(hDC, &tm);
            if (!GetOutlineTextMetrics(hDC, sizeof(MetricBuffer), potm))
                potm->otmSize = 0;
            SelectObject(hDC, hFontOld);
            GetObject(hFont, sizeof(lfReal), &lfReal);
            DeleteObject(hFont);
        }
        psz = GetFontSummary(hDC, &lfReal, &tm, potm);
        hFontOld = SelectObject(hDC, hGuiFont);
        Height = tm.tmHeight + tm.tmExternalLeading;
        TextOut(hDC, X, Y + Height, psz, lstrlen(psz));
        SelectObject(hDC, hFontOld);
        ++Index;
        if (Height + 12 > 60)
            Y += Height + 12;
        else
            Y += 60;

        lf.lfWeight = FW_BOLD;
        lf.lfItalic = FALSE;
        lf.lfUnderline = FALSE;
        lf.lfStrikeOut = FALSE;
        hFont = CreateFontIndirect(&lf);
        if (hFont)
        {
            hFontOld = SelectObject(hDC, hFont);
            TextOut(hDC, X, Y, texts[Index], lstrlen(texts[Index]));
            GetTextMetrics(hDC, &tm);
            if (!GetOutlineTextMetrics(hDC, sizeof(MetricBuffer), potm))
                potm->otmSize = 0;
            SelectObject(hDC, hFontOld);
            GetObject(hFont, sizeof(lfReal), &lfReal);
            DeleteObject(hFont);
        }
        psz = GetFontSummary(hDC, &lfReal, &tm, potm);
        hFontOld = SelectObject(hDC, hGuiFont);
        Height = tm.tmHeight + tm.tmExternalLeading;
        TextOut(hDC, X, Y + Height, psz, lstrlen(psz));
        SelectObject(hDC, hFontOld);
        ++Index;
        if (Height + 12 > 60)
            Y += Height + 12;
        else
            Y += 60;

        lf.lfWeight = FW_NORMAL;
        lf.lfItalic = TRUE;
        lf.lfUnderline = FALSE;
        lf.lfStrikeOut = FALSE;
        hFont = CreateFontIndirect(&lf);
        if (hFont)
        {
            hFontOld = SelectObject(hDC, hFont);
            TextOut(hDC, X, Y, texts[Index], lstrlen(texts[Index]));
            GetTextMetrics(hDC, &tm);
            if (!GetOutlineTextMetrics(hDC, sizeof(MetricBuffer), potm))
                potm->otmSize = 0;
            SelectObject(hDC, hFontOld);
            GetObject(hFont, sizeof(lfReal), &lfReal);
            DeleteObject(hFont);
        }
        psz = GetFontSummary(hDC, &lfReal, &tm, potm);
        hFontOld = SelectObject(hDC, hGuiFont);
        Height = tm.tmHeight + tm.tmExternalLeading;
        TextOut(hDC, X, Y + Height, psz, lstrlen(psz));
        SelectObject(hDC, hFontOld);
        ++Index;
        if (Height + 12 > 60)
            Y += Height + 12;
        else
            Y += 60;

        lf.lfWeight = FW_NORMAL;
        lf.lfItalic = FALSE;
        lf.lfUnderline = TRUE;
        lf.lfStrikeOut = FALSE;
        hFont = CreateFontIndirect(&lf);
        if (hFont)
        {
            hFontOld = SelectObject(hDC, hFont);
            TextOut(hDC, X, Y, texts[Index], lstrlen(texts[Index]));
            GetTextMetrics(hDC, &tm);
            if (!GetOutlineTextMetrics(hDC, sizeof(MetricBuffer), potm))
                potm->otmSize = 0;
            SelectObject(hDC, hFontOld);
            GetObject(hFont, sizeof(lfReal), &lfReal);
            DeleteObject(hFont);
        }
        psz = GetFontSummary(hDC, &lfReal, &tm, potm);
        hFontOld = SelectObject(hDC, hGuiFont);
        Height = tm.tmHeight + tm.tmExternalLeading;
        TextOut(hDC, X, Y + Height, psz, lstrlen(psz));
        SelectObject(hDC, hFontOld);
        ++Index;
        if (Height + 12 > 60)
            Y += Height + 12;
        else
            Y += 60;

        lf.lfWeight = FW_NORMAL;
        lf.lfItalic = FALSE;
        lf.lfUnderline = FALSE;
        lf.lfStrikeOut = TRUE;
        hFont = CreateFontIndirect(&lf);
        if (hFont)
        {
            hFontOld = SelectObject(hDC, hFont);
            TextOut(hDC, X, Y, texts[Index], lstrlen(texts[Index]));
            GetTextMetrics(hDC, &tm);
            if (!GetOutlineTextMetrics(hDC, sizeof(MetricBuffer), potm))
                potm->otmSize = 0;
            SelectObject(hDC, hFontOld);
            GetObject(hFont, sizeof(lfReal), &lfReal);
            DeleteObject(hFont);
        }
        psz = GetFontSummary(hDC, &lfReal, &tm, potm);
        hFontOld = SelectObject(hDC, hGuiFont);
        Height = tm.tmHeight + tm.tmExternalLeading;
        TextOut(hDC, X, Y + Height, psz, lstrlen(psz));
        SelectObject(hDC, hFontOld);

        EndPaint(hwnd, &ps);
    }

    DeleteObject(hGuiFont);
}

extern "C"
INT_PTR CALLBACK
DialogProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch (uMsg)
    {
        HANDLE_MSG(hwnd, WM_INITDIALOG, OnInitDialog);
        HANDLE_MSG(hwnd, WM_COMMAND, OnCommand);
        HANDLE_MSG(hwnd, WM_PAINT, OnPaint);
    }
    return 0;
}

extern "C"
INT WINAPI
WinMain(
    HINSTANCE   hInstance,
    HINSTANCE   hPrevInstance,
    LPSTR       lpCmdLine,
    INT         nCmdShow)
{
    g_hInstance = hInstance;
    DialogBox(hInstance, MAKEINTRESOURCE(1), NULL, DialogProc);
    return 0;
}
